% inverter model
clear all;
close all;

% define constants for lambda phage repressor
% all units should be 1/cell too
n_av = 6.022e23; % avogadro's number, molecules/mole
v_cell = 0.7e-15; % cell volume, liters
pops_i = 1; % mRNA transcription rate, 1/second
pops_i_max = 1; % max input pops, 1/second
rho = 1e-3; % leakiness parameter, unitless
leaky_i = rho*pops_i_max; % rate of spontaneous transcript generation, 1/second
k_transcription = pops_i+leaky_i;
k_spi = 0.17; % translation rate, 1/second
mu = 1/(60^2); % growth rate of cells, 1/second
pi_halflife = 2400; % repressor half-life, seconds
mi_halflife = 120; % mRNA half-life, seconds
k_d_pi = log(2)/pi_halflife + mu; % monomer degradation rate, 1/seconds
k_d_pipi = 0; % dimer degradation rate, 1/seconds
k_d_mi = log(2)/mi_halflife + mu; % mRNA degradation rate, 1/seconds
% assuming diffusion limit
k_on_pipido = 1e9/n_av/v_cell; % repressor binding rate, 1/(molecule*seconds)
k_on_pipi = k_on_pipido; % dimerization rate, 1/(molecule*seconds)
K_1 = (2e-8)*n_av*v_cell; % dimerization dissociation constant, molecules
%K_1 = 5e6; % dimerization dissociation constant, molecules
K_2 = (3e-9)*n_av*v_cell; % repressor binding dissociation constant, molecules
%K_2 = 5; % repressor binding dissociation constant, molecules
k_off_pipido = k_on_pipido*K_2; % repressor unbinding rate, 1/seconds
k_off_pipi = k_on_pipi*K_1; % dimer dissociation rate, 1/seconds
gn = 4.6e6; % bp in gn
K_5 = (10e-4)*n_av*v_cell; % dissociation constant for non-specific pi binding
%K_5 = 4e5; % dissociation constant for non-specific pi binding
k_on_ns_pi = (1e-6)*(1e9/n_av/v_cell); % non-specific pi binding, 1/(molecule*seconds)
k_off_ns_pi = k_on_ns_pi*K_5; % non-specific pi unbinding rate, 1/seconds
K_d_ns_pipi = K_5; % dissociation constant for non-specific pipi binding
k_on_ns_pipi = k_on_ns_pi; % non-specific pipi binding, 1/(molecule*seconds)
k_off_ns_pipi = k_off_ns_pi; % non-specific pipi unbinding rate, 1/seconds
gene_i = 10; % copies of input gene, molecules/cell
gene_o = 10; % copies of output gene, molecules/cell
pops_o_max = 1; % output mRNA transcription rate, 1/second
leaky_o = leaky_i; % output leakiness, 1/second

% create 'inverter' model object
Mobj = sbiomodel('inverter');

% add reaction objects
Robj1 = addreaction(Mobj, 'gi -> gi + mi'); % transcription
Robj2 = addreaction(Mobj, 'mi -> mi + pi'); % translation
Robj3 = addreaction(Mobj, 'pi + pi <-> pipi'); % pipiization
Robj4 = addreaction(Mobj, 'pipi + do <-> pipido'); % reversible binding of pipi
Robj5 = addreaction(Mobj, 'mi -> degraded_mi'); % mi degradation
Robj6 = addreaction(Mobj, 'pi -> degraded_pi'); % pi degradation
Robj7 = addreaction(Mobj, 'pipi -> degraded_pipi'); % pipi degradation
Robj8 = addreaction(Mobj, 'pi + gn <-> pign'); % non-specific binding of pi to gn
Robj9 = addreaction(Mobj, 'pipi + gn <-> pipign'); % non-specific binding of pipi to gn

% add kinetic law objects
Kobj1 = addkineticlaw(Robj1, 'MassAction');
Kobj2 = addkineticlaw(Robj2, 'MassAction');
Kobj3 = addkineticlaw(Robj3, 'MassAction');
Kobj4 = addkineticlaw(Robj4, 'MassAction');
Kobj5 = addkineticlaw(Robj5, 'MassAction');
Kobj6 = addkineticlaw(Robj6, 'MassAction');
Kobj7 = addkineticlaw(Robj7, 'MassAction');
Kobj8 = addkineticlaw(Robj8, 'MassAction');
Kobj9 = addkineticlaw(Robj9, 'MassAction');

% add parameter objects
Pobj1 = addparameter(Kobj1, 'k_transcription', k_transcription);
Pobj2 = addparameter(Kobj2, 'k_spi', k_spi);
Pobj3f = addparameter(Kobj3, 'k_on_pipi', k_on_pipi);
Pobj3r = addparameter(Kobj3, 'k_off_pipi', k_off_pipi);
Pobj4f = addparameter(Kobj4, 'k_on_pipido', k_on_pipido);
Pobj4r = addparameter(Kobj4, 'k_off_pipido', k_off_pipido);
% these are just arrays of variables so that i didn't have to type the same
% thing over and over
k_h_str = {'k_d_mi', 'k_d_pi', 'k_d_pipi'};
k_h_var = [k_d_mi, k_d_pi, k_d_pipi];
Pobj5 = addparameter(Kobj5, k_h_str{1}, k_h_var(1));
Pobj6 = addparameter(Kobj6, k_h_str{2}, k_h_var(2));
Pobj7 = addparameter(Kobj7, k_h_str{3}, k_h_var(3));
k_ns_pi_str = {'k_on_ns_pi', 'k_off_ns_pi'};
k_ns_pipi_str = {'k_on_ns_pipi', 'k_off_ns_pipi'};
k_ns_var = [k_on_ns_pi, k_off_ns_pi, k_on_ns_pipi, k_off_ns_pipi];
Pobj8f = addparameter(Kobj8, k_ns_pi_str{1}, k_ns_var(1));
Pobj8r = addparameter(Kobj8, k_ns_pi_str{2}, k_ns_var(2));
Pobj9f = addparameter(Kobj9, k_ns_pipi_str{1}, k_ns_var(3));
Pobj9r = addparameter(Kobj9, k_ns_pipi_str{2}, k_ns_var(4));

% set rate constant in kinetic law
pvn = 'ParameterVariableNames';
set(Kobj1, pvn, 'k_transcription');
set(Kobj2, pvn, 'k_spi');
k_pipi = {'k_on_pipi', 'k_off_pipi'};
set(Kobj3, pvn, k_pipi);
k_pipido = {'k_on_pipido', 'k_off_pipido'};
set(Kobj4, pvn, k_pipido);
set(Kobj5, pvn, k_h_str{1});
set(Kobj6, pvn, k_h_str{2});
set(Kobj7, pvn, k_h_str{3});
set(Kobj8, pvn, k_ns_pi_str);
set(Kobj9, pvn, k_ns_pipi_str);

% initial amounts for species
% defaults to 0
s = sbioselect(Mobj, 'Type', 'species', 'Name', 'gi');
s.InitialAmount = gene_i;
s = sbioselect(Mobj, 'Type', 'species', 'Name', 'do');
s.InitialAmount = gene_o;
s = sbioselect(Mobj, 'Type', 'species', 'Name', 'gn');
s.InitialAmount = gn;