% inverter model
clear all;
close all;

% define constants for lambda phage repressor
n_av = 6.022e23; % avogadro's number, molecules/mole
v_cell = 0.7e-15; % cell volume, liters
pops_i = 1; % mRNA transcription rate, 1/second
k_transcription = pops_i;
k_translation = 0.17; % translation rate, 1/second
mu = 1/(60^2); % growth rate of cells, 1/second
pi_hl = 2400; % repressor half-life, seconds
mrna_hl = 120; % mRNA half-life, seconds
k_h_monomer = log(2)/pi_hl + mu; % monomer degradation rate, 1/seconds
k_h_dimer = k_h_monomer; % dimer degradation rate, 1/seconds
k_h_mrna = log(2)/mrna_hl + mu; % mRNA degradation rate, 1/seconds
%assuming diffusion limit
k_on = 1e9/n_av/v_cell; % repressor binding rate, 1/(molecule*seconds)
k_on_dimer = k_on; % dimerization rate, 1/(molecule*seconds)
K_d_dimer = (2e-8)*n_av*v_cell; % dimerization dissociation constant, molecules
K_d = (3e-9)*n_av*v_cell; % repressor binding dissociation constant, molecules
k_off = k_on/K_d; % repressor unbinding rate, 1/seconds
k_off_dimer = k_on_dimer/K_d_dimer; % dimer dissociation rate, 1/seconds
gene_i = 1; % copies of input gene, molecules/cell
gene_o = 1; % copies of output gene, molecules/cell
pops_o_max = 1; % output mRNA transcription rate, 1/second

% create 'inverter' model object
Mobj = sbiomodel('inverter');

% add reaction objects
Robj1 = addreaction(Mobj, 'gene_input -> gene_input + mrna'); % transcription
Robj2 = addreaction(Mobj, 'mrna -> mrna + monomer'); % translation
Robj3 = addreaction(Mobj, 'monomer + monomer <-> dimer'); % dimerization
Robj4 = addreaction(Mobj, 'dimer + operator <-> complex'); % reversible binding of dimer
Robj5 = addreaction(Mobj, 'mrna -> null'); % mrna degradation
Robj6 = addreaction(Mobj, 'monomer -> null'); % monomer degradation
Robj7 = addreaction(Mobj, 'dimer -> null'); % dimer degradation

% add kinetic law objects
Kobj1 = addkineticlaw(Robj1, 'MassAction');
Kobj2 = addkineticlaw(Robj2, 'MassAction');
Kobj3 = addkineticlaw(Robj3, 'MassAction');
Kobj4 = addkineticlaw(Robj4, 'MassAction');
Kobj5 = addkineticlaw(Robj5, 'MassAction');
Kobj6 = addkineticlaw(Robj6, 'MassAction');
Kobj7 = addkineticlaw(Robj7, 'MassAction');

% add parameter objects
Pobj1 = addparameter(Kobj1, 'k_transcription', k_transcription);
Pobj2 = addparameter(Kobj2, 'k_translation', k_translation);
Pobj3f = addparameter(Kobj3, 'k_on_dimer', k_on_dimer);
Pobj3r = addparameter(Kobj3, 'k_off_dimer', k_off_dimer);
Pobj4f = addparameter(Kobj4, 'k_on', k_on);
Pobj4r = addparameter(Kobj4, 'k_off', k_off);
k_h_str = {'k_h_mrna', 'k_h_monomer', 'k_h_dimer'};
k_h_var = [k_h_mrna, k_h_monomer, k_h_dimer];
Pobj5 = addparameter(Kobj5, k_h_str{1}, k_h_var(1));
Pobj6 = addparameter(Kobj6, k_h_str{2}, k_h_var(2));
Pobj7 = addparameter(Kobj7, k_h_str{3}, k_h_var(3));

% set rate constant in kinetic law
pvn = 'ParameterVariableNames';
set(Kobj1, pvn, 'k_transcription');
set(Kobj2, pvn, 'k_translation');
k_dimer = {'k_on_dimer', 'k_off_dimer'};
set(Kobj3, pvn, k_dimer);
k_complex = {'k_on', 'k_off'};
set(Kobj4, pvn, k_complex);
set(Kobj5, pvn, k_h_str{1});
set(Kobj6, pvn, k_h_str{2});
set(Kobj7, pvn, k_h_str{3});

% initial amounts for species
% defaults to 0
s = sbioselect(Mobj, 'Type', 'species', 'Name', 'gene_input');
s.InitialAmount = gene_i;
s = sbioselect(Mobj, 'Type', 'species', 'Name', 'operator');
s.InitialAmount = gene_o;

% simulate with stochastic solver
state_mrna = Mobj.species(2);
state_monomer = Mobj.species(3);
state_dimer = Mobj.species(4);
state_operator = Mobj.species(5);
state_complex = Mobj.species(6);
StatesToLog = [state_operator, state_complex]; % choose which species to track
cs = getconfigset(Mobj);
set(cs, 'SolverType', 'ssa');
set(cs.RuntimeOptions, 'StatesToLog', StatesToLog);
cs.SolverOptions.LogDecimation = 1;
cs.StopTime = 10;
[t_ssa, x_ssa] = sbiosimulate(Mobj, cs);

% convert to PoPS
pops_o = [0:1:length(x_ssa)-1];
for index=1:length(x_ssa),
    operator_free = x_ssa(index,1);
    operator_bound = x_ssa(index,2);
    operator_total = operator_free + operator_bound;
    pops_o(index) = pops_o_max * operator_free / operator_total;
end

set(gcf, 'color', 'white');
figure(1);
plot(t_ssa,pops_o);
title('Inverter');
xlabel('Time (second)');
ylabel('PoPS out');
